<?php

namespace InSegment\ApiCore\Models;

use Illuminate\Support\Facades\DB;

use InSegment\ApiCore\Models\SliceRemap;
use InSegment\ApiCore\Exceptions\BaseSliceException;

use InSegment\ApiCore\Interfaces\SliceManagementInterface;
use InSegment\ApiCore\Traits\SliceManagementTrait;

use InSegment\ApiCore\Services\SliceMerger;
use InSegment\ApiCore\Services\ParseCreateTable;

class SliceOperation extends BaseOperation implements SliceManagementInterface
{
    use SliceManagementTrait;

    /**
     * Create new slice
     * 
     * @return \InSegment\ApiCore\Models\SliceOperation
     * @throws \InSegment\ApiCore\Exceptions\BaseSliceException
     */
    public static function newSlice()
    {
        $sliceUID = DB::selectOne('SELECT UUID_SHORT() as `uid`')->uid;
        if (!$sliceUID) {
            throw (new BaseSliceException(BaseSliceException::CODE_UNABLE_TO_GENERATE_UID))->compile();
        }
        
        return static::create(['uid' => $sliceUID]);
    }
    
    /**
     * Fillable attributes
     * 
     * @var array
     */
    protected $fillable = [
        'uid'
    ];
    
    /**
     * SliceMerger instance
     * 
     * @var \InSegment\ApiCore\Services\SliceMerger 
     */
    protected $sliceMerger;
    
    /**
     * Instance of show create table parser
     * 
     * @var \InSegment\ApiCore\Services\ParseCreateTable
     */
    protected $showCreateParser;
    
    /**
     * Counters for written and new records in the Slice (for estimates)
     *
     * @var array [
     *     'new' => [
     *         string $table => int,
     *         ...
     *     ]
     *     'written' => [
     *         string $table => int,
     *         ...
     *     ]
     * ] 
     */
    protected $counters = [];
    
    /**
     * Counters for written and new records in the Slice (for estimates)
     *
     * @param array $counters [
     *     'new' => [
     *         string $table => int,
     *         ...
     *     ]
     *     'written' => [
     *         string $table => int,
     *         ...
     *     ]
     * ] 
     */
    public function setCounters(array $counters)
    {
        $this->counters = $counters;
        
        return $this;
    }
    
    /**
     * Increase a counter for a table
     * 
     * @param string $type 'written' or 'new'
     * @param string $table
     * @param int $amount
     * @return $this
     */
    public function increaseTableCounter(string $type, string $table, int $amount)
    {
        if (isset($this->counters[$type][$table])) {
            $this->counters[$type][$table] += $amount;
        } else {
            $this->counters[$type][$table] = $amount;
        }
        
        return $this;
    }
    
    /**
     * Get table for remaps
     * 
     * @return string
     */
    public function getRemapsTable()
    {
        return (new SliceRemap([], $this->uid))->getTable();
    }
    
    /**
     * Get ParseCreateTable instance
     * 
     * @return \InSegment\ApiCore\Services\ParseCreateTable
     */
    public function getShowCreateParser()
    {
        return $this->showCreateParser ?? ($this->showCreateParser = ParseCreateTable::getInstance());
    }

    /**
     * Get SliceMerger instance
     * 
     * @return \InSegment\ApiCore\Services\SliceMerger
     */
    public function getSliceMerger()
    {
        return $this->sliceMerger ?? ($this->sliceMerger = new SliceMerger($this, $this->getShowCreateParser(), [SliceMerger::TEMP_TABLE_TYPE_INSERT]));
    }

    /**
     * Get slice operation UID
     * 
     * @return string
     */
    public function getUID(): string
    {
        return $this->uid;
    }
    
    /**
     * Get count of records was written for table
     * 
     * @param string $table
     * @return int
     */
    public function getWrittenCount(string $table): int
    {
        return $this->counters['written'][$table];
    }
    
    /**
     * Get count of records was inserted into table
     * 
     * @param string $table
     * @return int
     */
    public function getNewCount(string $table): int
    {
        return $this->counters['new'][$table];
    }
}
